@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2001-2002  Nokia Corporation
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2009  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2010  Marcel Holtmann <marcel@holtmann.org>
  *  Copyright (C) 2002-2003  Stephen Crane <steve.crane@rococosoft.com>
  *
  *
@@ -43,7 +43,7 @@
 #include <netinet/in.h>
 
 #include "sdpd.h"
-#include "logging.h"
+#include "log.h"
 
 #define MIN(x, y) ((x) < (y)) ? (x): (y)
 
@@ -86,11 +86,19 @@ static uint32_t sdp_cstate_alloc_buf(sdp_buf_t *buf)
 
 /* Additional values for checking datatype (not in spec) */
 #define SDP_TYPE_UUID	0xfe
-#define SDP_TYPE_ANY	0xff
+#define SDP_TYPE_ATTRID	0xff
+
+struct attrid {
+	uint8_t dtd;
+	union {
+		uint16_t uint16;
+		uint32_t uint32;
+	};
+};
 
 /*
  * Generic data element sequence extractor. Builds
- * a list whose elements are those found in the 
+ * a list whose elements are those found in the
  * sequence. The data type of elements found in the
  * sequence is returned in the reference pDataType
  */
@@ -101,10 +109,10 @@ static int extract_des(uint8_t *buf, int len, sdp_list_t **svcReqSeq, uint8_t *p
 	short numberOfElements = 0;
 	int seqlen = 0;
 	sdp_list_t *pSeq = NULL;
-	uint8_t dataType = expectedType;
+	uint8_t dataType;
 	int status = 0;
 	const uint8_t *p;
-	int bufsize;
+	size_t bufsize;
 
 	scanned = sdp_extract_seqtype(buf, len, &seqType, &data_size);
 
@@ -122,14 +130,11 @@ static int extract_des(uint8_t *buf, int len, sdp_list_t **svcReqSeq, uint8_t *p
 		char *pElem = NULL;
 		int localSeqLength = 0;
 
-		if (bufsize < (int) sizeof(uint8_t)) {
+		if (bufsize < sizeof(uint8_t)) {
 			SDPDBG("->Unexpected end of buffer");
 			goto failed;
 		}
 
-		if (dataType == SDP_TYPE_ANY)
-			expectedType = *p;
-
 		dataType = *p;
 
 		SDPDBG("Data type: 0x%02x", dataType);
@@ -139,7 +144,12 @@ static int extract_des(uint8_t *buf, int len, sdp_list_t **svcReqSeq, uint8_t *p
 				SDPDBG("->Unexpected Data type (expected UUID_ANY)");
 				goto failed;
 			}
-		} else if (expectedType != SDP_TYPE_ANY && dataType != expectedType) {
+		} else if (expectedType == SDP_TYPE_ATTRID &&
+				(dataType != SDP_UINT16 && dataType != SDP_UINT32)) {
+			SDPDBG("->Unexpected Data type (expected 0x%02x or 0x%02x)",
+								SDP_UINT16, SDP_UINT32);
+			goto failed;
+		} else if (expectedType != SDP_TYPE_ATTRID && dataType != expectedType) {
 			SDPDBG("->Unexpected Data type (expected 0x%02x)", expectedType);
 			goto failed;
 		}
@@ -149,13 +159,21 @@ static int extract_des(uint8_t *buf, int len, sdp_list_t **svcReqSeq, uint8_t *p
 			p += sizeof(uint8_t);
 			seqlen += sizeof(uint8_t);
 			bufsize -= sizeof(uint8_t);
-			if (bufsize < (int) sizeof(uint16_t)) {
+			if (bufsize < sizeof(uint16_t)) {
 				SDPDBG("->Unexpected end of buffer");
 				goto failed;
 			}
 
-			pElem = malloc(sizeof(uint16_t));
-			bt_put_unaligned(ntohs(bt_get_unaligned((uint16_t *)p)), (uint16_t *)pElem);
+			if (expectedType == SDP_TYPE_ATTRID) {
+				struct attrid *aid;
+				aid = malloc(sizeof(struct attrid));
+				aid->dtd = dataType;
+				bt_put_unaligned(ntohs(bt_get_unaligned((uint16_t *)p)), (uint16_t *)&aid->uint16);
+				pElem = (char *) aid;
+			} else {
+				pElem = malloc(sizeof(uint16_t));
+				bt_put_unaligned(ntohs(bt_get_unaligned((uint16_t *)p)), (uint16_t *)pElem);
+			}
 			p += sizeof(uint16_t);
 			seqlen += sizeof(uint16_t);
 			bufsize -= sizeof(uint16_t);
@@ -169,8 +187,16 @@ static int extract_des(uint8_t *buf, int len, sdp_list_t **svcReqSeq, uint8_t *p
 				goto failed;
 			}
 
-			pElem = malloc(sizeof(uint32_t));
-			bt_put_unaligned(ntohl(bt_get_unaligned((uint32_t *)p)), (uint32_t *)pElem);
+			if (expectedType == SDP_TYPE_ATTRID) {
+				struct attrid *aid;
+				aid = malloc(sizeof(struct attrid));
+				aid->dtd = dataType;
+				bt_put_unaligned(ntohl(bt_get_unaligned((uint32_t *)p)), (uint32_t *)&aid->uint32);
+				pElem = (char *) aid;
+			} else {
+				pElem = malloc(sizeof(uint32_t));
+				bt_put_unaligned(ntohl(bt_get_unaligned((uint32_t *)p)), (uint32_t *)pElem);
+			}
 			p += sizeof(uint32_t);
 			seqlen += sizeof(uint32_t);
 			bufsize -= sizeof(uint32_t);
@@ -275,8 +301,8 @@ static int sdp_cstate_get(uint8_t *buffer, size_t len,
  * specified in the "search pattern" must be present in the
  * "target pattern". Here "search pattern" is the set of UUIDs
  * specified by the service discovery client and "target pattern"
- * is the set of UUIDs present in a service record. 
- * 
+ * is the set of UUIDs present in a service record.
+ *
  * Return 1 if each and every UUID in the search
  * pattern exists in the target pattern, 0 if the
  * match succeeds and -1 on error.
@@ -341,7 +367,7 @@ static int service_search_req(sdp_req_t *req, sdp_buf_t *buf)
 	mlen = scanned + sizeof(uint16_t) + 1;
 	// ensure we don't read past buffer
 	if (plen < mlen || plen != mlen + *(uint8_t *)(pdata+sizeof(uint16_t))) {
-		status = SDP_INVALID_SYNTAX;
+		status = SDP_INVALID_PDU_SIZE;
 		goto done;
 	}
 
@@ -394,7 +420,7 @@ static int service_search_req(sdp_req_t *req, sdp_buf_t *buf)
 			sdp_record_t *rec = (sdp_record_t *) list->data;
 
 			SDPDBG("Checking svcRec : 0x%x", rec->handle);
-				
+
 			if (sdp_match_uuid(pattern, rec->pattern) > 0 &&
 					sdp_check_access(rec->handle, &req->device)) {
 				rsp_count++;
@@ -403,7 +429,7 @@ static int service_search_req(sdp_req_t *req, sdp_buf_t *buf)
 				handleSize += sizeof(uint32_t);
 			}
 		}
-		
+
 		SDPDBG("Match count: %d", rsp_count);
 
 		buf->data_size += handleSize;
@@ -499,9 +525,8 @@ static int service_search_req(sdp_req_t *req, sdp_buf_t *buf)
 		}
 	}
 
-done:	
-	if (cstate)
-		free(cstate);
+done:
+	free(cstate);
 	if (pattern)
 		sdp_list_free(pattern, free);
 
@@ -515,8 +540,10 @@ done:
  * requested identifiers are present in the PDU form of
  * the request
  */
-static int extract_attrs(sdp_record_t *rec, sdp_list_t *seq, uint8_t dtd, sdp_buf_t *buf)
+static int extract_attrs(sdp_record_t *rec, sdp_list_t *seq, sdp_buf_t *buf)
 {
+	sdp_buf_t pdu;
+
 	if (!rec)
 		return SDP_INVALID_RECORD_HANDLE;
 
@@ -526,24 +553,25 @@ static int extract_attrs(sdp_record_t *rec, sdp_list_t *seq, uint8_t dtd, sdp_bu
 		SDPDBG("NULL attribute descriptor");
 	}
 
-	SDPDBG("AttrDataType : %d", dtd);
-
 	if (seq == NULL) {
 		SDPDBG("Attribute sequence is NULL");
 		return 0;
 	}
-	if (dtd == SDP_UINT16)
-		for (; seq; seq = seq->next) {
-			uint16_t attr = bt_get_unaligned((uint16_t *)seq->data);
+
+	sdp_gen_record_pdu(rec, &pdu);
+
+	for (; seq; seq = seq->next) {
+		struct attrid *aid = seq->data;
+
+		SDPDBG("AttrDataType : %d", aid->dtd);
+
+		if (aid->dtd == SDP_UINT16) {
+			uint16_t attr = bt_get_unaligned((uint16_t *)&aid->uint16);
 			sdp_data_t *a = (sdp_data_t *)sdp_data_get(rec, attr);
 			if (a)
 				sdp_append_to_pdu(buf, a);
-		}
-	else if (dtd == SDP_UINT32) {
-		sdp_buf_t pdu;
-		sdp_gen_record_pdu(rec, &pdu);
-		for (; seq; seq = seq->next) {
-			uint32_t range = bt_get_unaligned((uint32_t *)seq->data);
+		} else if (aid->dtd == SDP_UINT32) {
+			uint32_t range = bt_get_unaligned((uint32_t *)&aid->uint32);
 			uint16_t attr;
 			uint16_t low = (0xffff0000 & range) >> 16;
 			uint16_t high = 0x0000ffff & range;
@@ -568,13 +596,16 @@ static int extract_attrs(sdp_record_t *rec, sdp_list_t *seq, uint8_t dtd, sdp_bu
 			data = sdp_data_get(rec, high);
 			if (data)
 				sdp_append_to_pdu(buf, data);
+		} else {
+			error("Unexpected data type : 0x%x", aid->dtd);
+			error("Expect uint16_t or uint32_t");
+			free(pdu.data);
+			return SDP_INVALID_SYNTAX;
 		}
-		free(pdu.data);
-	} else {
-		error("Unexpected data type : 0x%x", dtd);
-		error("Expect uint16_t or uint32_t");
-		return SDP_INVALID_SYNTAX;
 	}
+
+	free(pdu.data);
+
 	return 0;
 }
 
@@ -624,7 +655,7 @@ static int service_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 	}
 
 	/* extract the attribute list */
-	scanned = extract_des(pdata, data_left, &seq, &dtd, SDP_TYPE_ANY);
+	scanned = extract_des(pdata, data_left, &seq, &dtd, SDP_TYPE_ATTRID);
 	if (scanned == -1) {
 		status = SDP_INVALID_SYNTAX;
 		goto done;
@@ -636,7 +667,7 @@ static int service_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 	mlen = scanned + sizeof(uint32_t) + sizeof(uint16_t) + 1;
 	// ensure we don't read past buffer
 	if (plen < mlen || plen != mlen + *(uint8_t *)pdata) {
-		status = SDP_INVALID_SYNTAX;
+		status = SDP_INVALID_PDU_SIZE;
 		goto done;
 	}
 
@@ -652,11 +683,20 @@ static int service_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 	SDPDBG("SvcRecHandle : 0x%x", handle);
 	SDPDBG("max_rsp_size : %d", max_rsp_size);
 
-	/* 
+	/*
+	 * Check that max_rsp_size is within valid range
+	 * a minimum size of 0x0007 has to be used for data field
+	 */
+	if (max_rsp_size < 0x0007) {
+		status = SDP_INVALID_SYNTAX;
+		goto done;
+	}
+
+	/*
 	 * Calculate Attribute size acording to MTU
 	 * We can send only (MTU - sizeof(sdp_pdu_hdr_t) - sizeof(sdp_cont_state_t))
 	 */
-	max_rsp_size = MIN(max_rsp_size, req->mtu - sizeof(sdp_pdu_hdr_t) - 
+	max_rsp_size = MIN(max_rsp_size, req->mtu - sizeof(sdp_pdu_hdr_t) -
 			sizeof(uint32_t) - SDP_CONT_STATE_SIZE - sizeof(uint16_t));
 
 	/* pull header for AttributeList byte count */
@@ -687,7 +727,7 @@ static int service_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 		}
 	} else {
 		sdp_record_t *rec = sdp_record_find(handle);
-		status = extract_attrs(rec, seq, dtd, buf);
+		status = extract_attrs(rec, seq, buf);
 		if (buf->data_size > max_rsp_size) {
 			sdp_cont_state_t newState;
 
@@ -713,8 +753,7 @@ static int service_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 	buf->buf_size += sizeof(uint16_t);
 
 done:
-	if (cstate)
-		free(cstate);
+	free(cstate);
 	if (seq)
 		sdp_list_free(seq, free);
 	if (status)
@@ -775,7 +814,7 @@ static int service_search_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 	}
 
 	/* extract the attribute list */
-	scanned = extract_des(pdata, data_left, &seq, &dtd, SDP_TYPE_ANY);
+	scanned = extract_des(pdata, data_left, &seq, &dtd, SDP_TYPE_ATTRID);
 	if (scanned == -1) {
 		status = SDP_INVALID_SYNTAX;
 		goto done;
@@ -788,7 +827,7 @@ static int service_search_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 
 	plen = ntohs(((sdp_pdu_hdr_t *)(req->buf))->plen);
 	if (plen < totscanned || plen != totscanned + *(uint8_t *)pdata) {
-		status = SDP_INVALID_SYNTAX;
+		status = SDP_INVALID_PDU_SIZE;
 		goto done;
 	}
 
@@ -808,7 +847,7 @@ static int service_search_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 	tmpbuf.buf_size = USHRT_MAX;
 	memset(tmpbuf.data, 0, USHRT_MAX);
 
-	/* 
+	/*
 	 * Calculate Attribute size acording to MTU
 	 * We can send only (MTU - sizeof(sdp_pdu_hdr_t) - sizeof(sdp_cont_state_t))
 	 */
@@ -826,7 +865,7 @@ static int service_search_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 			if (sdp_match_uuid(pattern, rec->pattern) > 0 &&
 					sdp_check_access(rec->handle, &req->device)) {
 				rsp_count++;
-				status = extract_attrs(rec, seq, dtd, &tmpbuf);
+				status = extract_attrs(rec, seq, &tmpbuf);
 
 				SDPDBG("Response count : %d", rsp_count);
 				SDPDBG("Local PDU size : %d", tmpbuf.data_size);
@@ -897,10 +936,8 @@ static int service_search_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 	}
 
 done:
-	if (cstate)
-		free(cstate);
-	if (tmpbuf.data)
-		free(tmpbuf.data);
+	free(cstate);
+	free(tmpbuf.data);
 	if (pattern)
 		sdp_list_free(pattern, free);
 	if (seq)
@@ -1008,20 +1045,33 @@ void handle_request(int sk, uint8_t *data, int len)
 	sdp_req_t req;
 
 	size = sizeof(sa);
-	if (getpeername(sk, (struct sockaddr *) &sa, &size) < 0)
+	if (getpeername(sk, (struct sockaddr *) &sa, &size) < 0) {
+		error("getpeername: %s", strerror(errno));
 		return;
+	}
 
-	if (sa.l2_family == AF_BLUETOOTH) { 
+	if (sa.l2_family == AF_BLUETOOTH) {
 		struct l2cap_options lo;
+
 		memset(&lo, 0, sizeof(lo));
 		size = sizeof(lo);
-		getsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &lo, &size);
+
+		if (getsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &lo, &size) < 0) {
+			error("getsockopt: %s", strerror(errno));
+			return;
+		}
+
 		bacpy(&req.bdaddr, &sa.l2_bdaddr);
 		req.mtu = lo.omtu;
 		req.local = 0;
 		memset(&sa, 0, sizeof(sa));
 		size = sizeof(sa);
-		getsockname(sk, (struct sockaddr *) &sa, &size);
+
+		if (getsockname(sk, (struct sockaddr *) &sa, &size) < 0) {
+			error("getsockname: %s", strerror(errno));
+			return;
+		}
+
 		bacpy(&req.device, &sa.l2_bdaddr);
 	} else {
 		bacpy(&req.device, BDADDR_ANY);
